import javafx.application.Application;
import javafx.geometry.HPos;
import javafx.geometry.Insets;
import javafx.geometry.Pos;
import javafx.scene.Group;
import javafx.scene.Scene;
import javafx.scene.canvas.Canvas;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.control.*;
import javafx.scene.control.ScrollPane.ScrollBarPolicy;
import javafx.scene.layout.*;
import javafx.stage.Modality;
import javafx.stage.Stage;


import javafx.stage.StageStyle;
import javafx.stage.Window;
import javafx.scene.layout.GridPane;
import javafx.scene.text.Text;
import javafx.event.*;






public class GUI extends Application {
	
	Klauselmenge klauseln;

	String gueltVariablen;
	boolean gueltig;
	Canvas baum;
	static GraphicsContext gc;
	

	
    public static void main(String[] args) {
        launch(args);   
    }
    
    public void start(Stage primaryStage) {
    	
    	//Mengen initiieren
        gueltVariablen = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
        klauseln = new Klauselmenge();
        gueltig = false;
        

        //GUI Layout bestimmen
        /*
         * Es wird ein Grid Layout verwendet
         * Oben Formelanzeige und Button fuer neue Formeleingabe, unten Canvas um dem Baum zu zeichnen.
         */
        
        primaryStage.setTitle("Davis-Putnam");        
        final GridPane gridPane = new GridPane();
        gridPane.setPadding(new Insets(10, 10, 10, 10));
        gridPane.setHgap(10); gridPane.setVgap(10);
         
        
        
        final Label formel = new Label("Formelformat : {a,b,c},{!t,!p,Q},{T}");
        
        
       
        
        ScrollPane formelScroll = new ScrollPane();
        formelScroll.setHbarPolicy(ScrollBarPolicy.AS_NEEDED);
        formelScroll.setContent(formel);
        
        Button neueFormel = new Button("Neu");
        neueFormel.setOnAction(new EventHandler<ActionEvent>() {
        	//Buttonaktion definieren
            public void handle(ActionEvent event) {
                System.out.println("Benutzer will neue Formel eingeben");
                //Eingabefenster erzeugen
                FormelEingabe prompt = new FormelEingabe(gridPane.getScene().getWindow());
                System.out.println("Eingabe: " + prompt.getResult());
                
                //Pruefe, ob Formel in KNF (Mengenschreibweise)
                formel.setText(checkString(prompt.getResult()));
                
                //Baum malen, falls Formeleingabe gueltig
                
                /*
                 * 
                 * Hier wird Davis-Putnam Algorithmus eingesetzt
                 * 
                 */
                System.out.println("Eingabe fertig. gueltig: "+ gueltig);
                gc.fill();
                if (gueltig) {
                	
                	Anweisungsbaum ergebnis = DavisPutnam.algorithmus(klauseln);
                	
                	traverseAndPaint(ergebnis);
                }
            }
        });
        
        GridPane.setHalignment(neueFormel, HPos.CENTER);
        
        
        baum = new Canvas();
        baum.setHeight(300);
        baum.setWidth(500);
        gc = baum.getGraphicsContext2D();
    
        

        ScrollPane baumScroll = new ScrollPane();
        baumScroll.setHbarPolicy(ScrollBarPolicy.ALWAYS);
        baumScroll.setContent(baum);
       
         
        gridPane.setGridLinesVisible(false);
         
        
        
        ColumnConstraints formelConstraint = new ColumnConstraints();
        formelConstraint.setMinWidth(200);
        formelConstraint.setPercentWidth(90);
        
        ColumnConstraints neueFormelConstraint = new ColumnConstraints();
        neueFormelConstraint.setMaxWidth(100);
        neueFormelConstraint.setPercentWidth(10);
        
        RowConstraints formelConstraintR = new RowConstraints();
        formelConstraintR.setMaxHeight(50);
        formelConstraintR.setPercentHeight(10);
        
        RowConstraints baumConstraintR = new RowConstraints();
        baumConstraintR.setMinHeight(300);
        baumConstraintR.setPercentHeight(90);
        
        gridPane.getColumnConstraints().addAll(formelConstraint, neueFormelConstraint);
        gridPane.getRowConstraints().addAll(formelConstraintR, baumConstraintR);
        
        //GridPane.setColumnSpan(baum, 2);
        gridPane.add(formelScroll, 0, 0); 
        gridPane.add(neueFormel, 1, 0);
        gridPane.add(baumScroll, 0, 1);
        
        
        

        primaryStage.setScene(new Scene(gridPane, 600, 400));
        primaryStage.show();
        

        
    }
    
    private void traverseAndPaint(Anweisungsbaum b){
    	//Zeichne Baum
    	
    	System.out.println("Anzahl Leafs : "+ b.getNumberLeafs(b));
    	System.out.println("Links/Rechts Verteilung:");
    	if (b.rechteRegel != null){System.out.println("Hat rechts :" +b.getNumberLeafs(b.rechteRegel));}else{System.out.println("Hat keine Rechts !");}
    	if (b.linkeRegel != null){System.out.println("Hat links :" +b.getNumberLeafs(b.linkeRegel));}
    	
    	
    	
    	
    	Text text = new Text(b.aktKlauseln.toString());
    	System.out.println(b.aktKlauseln.toString());
        new Scene(new Group(text));
        text.applyCss(); 
        double widthText = text.getLayoutBounds().getWidth();
        
    	int xBreite = 200;
        int width = xBreite*(b.getNumberLeafs(b)+1);
        width += (int)widthText;
        
        
    	gc.clearRect(0, 0, baum.getWidth(), baum.getHeight());
    	
    	
    	System.out.println("Width: "+ width);
    	int height = 150 + b.getDepth()*80;
    	System.out.println("Drawing...");
    	baum.setWidth(width);
    	baum.setHeight(height);
    	
    	//Startposition mittels Leafverteilung whlen
    	Anweisungsbaum c = b.getFirstSplitNode(b);
    	int posX;
    	if (c != null){
    		System.out.println("First Spit Node Verteilung: "+ c.getNumberLeafs(c.linkeRegel) + " r: " + c.getNumberLeafs(c.rechteRegel) );
    		posX = (c.getNumberLeafs(c.linkeRegel)+1)*xBreite;
    	}else{
    		System.out.println("Kein Split");
    		posX = width/2;
    	}
    	
    	
    	int posY = 10;
    	
    	//Formelmenge zeichnen
    	
    	recPaint(b,posX,posY);    	
    }
    
    private void recPaint(Anweisungsbaum b, int posX, int posY){
    	//System.out.println("Aktuell zu zeichnen : "+b.aktKlauseln.toString() );
    	System.out.println("is top or bottom? " + b.aktKlauseln.getIsTop()+" "+b.aktKlauseln.getIsBottom());
    	
    	//System.out.println("Teilbaum hat Tiefe: "+b.getDepth() );
    	int yLength = 60;
    	
    	
    	
    	
    	    	
    	//Fall 1: Top    	
    	if(b.regel.equals("TOP")){
    		gc.strokeLine(posX, posY, posX, posY+yLength/2);
        	gc.fillText("1", posX-5, posY+yLength/2+15);
        	return;
    	}
    	
    	//Fall 2: Bottom
    	if(b.regel.equals("BOTTOM")){
    		gc.strokeLine(posX, posY, posX, posY+yLength/2);
        	gc.fillText("0", posX-5, posY+yLength/2+15);
        	return;
    	}
    	
    	//Fall 3: Keine Rechte Regel    	
    	
    	System.out.println("hat rechte regel: "+b.rechteRegel);
    	System.out.println("hat linke regel: "+b.linkeRegel);
    	
    	
    	if (b.rechteRegel == null && b.linkeRegel != null){
    
    		
    		gc.strokeLine(posX, posY, posX, posY+yLength);    		
        	gc.strokeOval(posX-5, posY+yLength, 10, 10);
        	
        	if (!b.regel.equals("SUBSUMPTION")){
        		gc.fillText(b.regel + " \u03C6(" + b.bezeichner +")= "+ b.belegung, posX-85, posY+yLength+10);
        	} else{
        		gc.fillText(b.regel  + " "+ b.bezeichner , posX+10, posY+yLength-3);
        	}        	   
        	gc.fillText(b.aktKlauseln.toString(), posX+10, posY+yLength + 10);
        	
        	recPaint(b.linkeRegel, posX, posY+10+yLength);
        	return;
    	}
    	
    	//Fall 4: Split
    	
    	if (b.linkeRegel != null && b.rechteRegel != null){
    		

    		
    		int xBreite = 200;
    		int xVerschiebungL = (b.getNumberLeafs(b.linkeRegel)*xBreite)/2;
    		int xVerschiebungR = (b.getNumberLeafs(b.rechteRegel)*xBreite)/2;
    		
    		Anweisungsbaum c = b.getFirstSplitNode(b.linkeRegel);
        	if (c != null){
        		System.out.println("First Spit Node Verteilung: "+ c.getNumberLeafs(c.linkeRegel) + " r: " + c.getNumberLeafs(c.rechteRegel) );
        		System.out.println("Vorher: "+ xVerschiebungL);
        		xVerschiebungL = (c.getNumberLeafs(c.rechteRegel))*xBreite;
        		System.out.println("Verndere Verschiebung nach Verteilung...");
        		System.out.println("Nachher: "+ xVerschiebungL);
        	}else{
        		System.out.println("Kein Split Links mehr");
        	}
        	
        	c = b.getFirstSplitNode(b.rechteRegel);
        	if (c != null){
        		System.out.println("First Spit Node Verteilung: "+ c.getNumberLeafs(c.linkeRegel) + " r: " + c.getNumberLeafs(c.rechteRegel) );
        		xVerschiebungR = (c.getNumberLeafs(c.linkeRegel))*xBreite;
        	}else{
        		System.out.println("Kein Split Rechts mehr");
        	}
    		
    		System.out.println("Split zeichnen");
    		
    		System.out.println("Verschiebung um -: "+ xVerschiebungL );
    		System.out.println("Verschiebung um +: "+ xVerschiebungR );
   
    		//Linker Teilbaum zeichnen
    		gc.strokeLine(posX, posY, posX-xVerschiebungL, posY+yLength+10);
    		gc.strokeOval(posX-xVerschiebungL-5, posY+yLength+10, 10, 10);
    		gc.fillText(b.regel + " \u03C6(" + b.bezeichner +")= 0", posX-xVerschiebungL-85, posY+yLength+20);
    		gc.fillText(b.aktKlauseln.toString(), posX-xVerschiebungL+10, posY+yLength + 20);
    		
    		
    		
    		//Rechter Teilbaum zeichnen
    		gc.strokeLine(posX, posY, posX+xVerschiebungR, posY+yLength+10);
    		gc.strokeOval(posX+xVerschiebungR-5, posY+yLength+10, 10, 10);
    		gc.fillText(b.regel + " \u03C6(" + b.bezeichner +")= 1", posX+xVerschiebungR-85, posY+yLength+20);
    		gc.fillText(b.aktKlauseln2.toString(), posX+xVerschiebungR+10, posY+yLength + 20);
    		
    		
    		
	    	recPaint(b.linkeRegel, posX-xVerschiebungL, posY+yLength+20);
	    	
	    	recPaint(b.rechteRegel, posX+xVerschiebungR, posY+yLength+20);
    		}
    		
    }
    	
    	
    	   	

    			
 
    	
    
    private String checkString(String eingabe){
    	/*
    	 * Prueft, ob Eingabe eine gueltige KNF ist
    	 * Format {a,b,c],{d,c,d} usw.
    	 * For Schleife geht String durch und wechselt Zustände
    	 * 
    	 * 1 == Start
    	 * 2 == Offene Klammer
    	 * 3 == Negierungszeichen
    	 * 4 == Variable
    	 * 5 == Schliessende Klammer
    	 * 
    	 */
    	String result = "";
    	int state = 1; 

    	//BUG: Bei keiner Eingabe (oder schliessen des leeren Dialogs) entsteht Exception
    	if (eingabe.isEmpty()){
    		return "Leere Eingabe";
    	}
    	if (eingabe.charAt(0) != "{".charAt(0) ) {
    		return "Ungueltige Eingabe";
    	}
    	
    	//Gehe String durch und erzeuge richtige Mengen, und den richtigen darzustellenden String   
    	klauseln = new Klauselmenge();
    	Klausel tempKlausel = new Klausel();
    	for (int i = 0; i < eingabe.length(); i++){
    	    char c = eingabe.charAt(i);   
    	    
    	    /*
    	    System.out.println("__________________________________________");
    	    System.out.println("Gerade betrachtet: " + c);
    	    System.out.println("State: " + state);
    	    System.out.println("Bis jetzt zusammengebauter String: " + result);
    	    System.out.println("Bis jetzt gebaute Klauselmenge: " + result);
    	    */
    	    
    	    
    	    if (state == 1 && c == "{".charAt(0)){
    	    	//Eine neue Klausel wurde gefunden, naechster Zustand"
    	    	state = 2;
    	    	tempKlausel = new Klausel();
    	    	result += "(";
    	    	
    	    } else{
    	    
    	    if (state == 2 && (  c == "!".charAt(0) ) )   {
    	    	//Negierung wurde gefunden
    	    	state = 3;
    	    	result += "\u00AC";
    	    				
    	    		
    	    	
    	    } else{
    	    	
    	    if (state == 2 && (  gueltVariablen.contains(""+c) ) ){
    	    	
    	    	//Variable wurde gefunden"
    	    	state = 4;
    	    	result += String.valueOf(c);
    	    	Variable a = new Variable(String.valueOf(c),false);
    	    	tempKlausel.addVariable(a);
    	    	
    	    } else{
    	    	
    	    if (state == 3 && (  gueltVariablen.contains(""+c) ) ){
    	    	
    	    	//Variable (negiert) wurde gefunden
        	    state = 4;
        	    result += String.valueOf(c);
        	    Variable a = new Variable(String.valueOf(c),true);
        	    tempKlausel.addVariable(a);
    	    	
        	} else{
        		
        	if (state == 4 && (  c == ",".charAt(0) ) ){
        	    	
        		//Oder gefunden
            	state = 2;
            	result += "\u2228";
        	    	
            } else {
            	
            if (state == 4 && (  c == "}".charAt(0) ) ){
        	    	
            	//Menge geschlossen
            	//System.out.println(tempKlausel.toString());
            	klauseln.addKlausel(tempKlausel);
            	result += ")";
                state = 5;  
                
            } else{
            
            if (state == 5 && (  c == ",".charAt(0) ) ){        	    	
            	//Und gefunden
                state = 1;  
                result += "\u2227";
            } else {gueltig = false;return "Ungültige Eingabe";}
            	
            	
            	
            } } } } }     
    	    }
    	    
    	    
    	    
    	    
    	}
    	System.out.println("result: "+result.toString());
    	gueltig = true;
    	return result;
    }

    
    
 
    
    class FormelEingabe {
    	/*
    	 * Dialogfenster, um eine neue Formel einzugeben.
    	 */
        private String result;

        FormelEingabe(Window owner) {
          final Stage dialog = new Stage();

          dialog.setTitle("Neue Formel eingeben:");
          dialog.initOwner(owner);
          dialog.initStyle(StageStyle.UTILITY);
          dialog.initModality(Modality.WINDOW_MODAL);
          dialog.setHeight(100);
          dialog.setWidth(400);
          dialog.setX(owner.getX());
          dialog.setY(owner.getY());

          final TextField textField = new TextField();
          final Button submitButton = new Button("Ok");
          submitButton.setDefaultButton(true);
          submitButton.setOnAction(new EventHandler<ActionEvent>() {
            public void handle(ActionEvent t) {
              dialog.close();
            }
          });
          textField.setMinHeight(TextField.USE_PREF_SIZE);

          final VBox layout = new VBox(10);
          layout.setAlignment(Pos.CENTER_RIGHT);
          layout.setStyle("-fx-background-color: azure; -fx-padding: 10;");
          layout.getChildren().setAll(
            textField, 
            submitButton
          );

          dialog.setScene(new Scene(layout));
          dialog.showAndWait();

          result = textField.getText();
        }

        private String getResult() {
          return result;
        }
      }


}

