import java.util.ArrayList;
import java.util.HashSet;
import java.util.ListIterator;


public class Klausel{
	/*
	 * Klasse zur Verwaltung einer KNF-Klausel.
	 */
	
	private ArrayList<Variable> variablen = new ArrayList<Variable>();
	
	public Klausel (ArrayList<Variable> vars){
		/*
		 * Konstruktor. Erstellt eine Klausel mit einer deep copy der Variablen.
		 */
		ListIterator<Variable> litr = vars.listIterator();
		while (litr.hasNext()) {
			Variable element = litr.next();
			String bez = element.getBezeichner();
			int bel = element.getBelegung();
			boolean neg = element.getNegiert();
			
			Variable neu = new Variable(bez,neg);
			neu.setBelegung(bel);
			variablen.add(neu);			
		}
		
	}
	
	public Klausel(){}
	

	public int eval(){
		/*
		 * Gibt Wahrheitswert 1, wenn eine Variable die Klausel erfüllt (Disjunktion). 
		 * Falls mindestens eine Variable nicht belegt ist und keine 1, wird -1 zurückgegeben.
		 * Sonst wird 0 zurückgegeben. 
		 */
		
		ListIterator<Variable> litr = variablen.listIterator();
		boolean eineNichtBelegt = false;
		
		while (litr.hasNext()) {
			Variable element = litr.next();
			//System.out.println("Betrachte Variable "+element.toString()+"... Bewertung: "+element.eval());
			if (element.eval() == 1){
				
				return 1;
			} else{
				if (element.eval() == -1){
					eineNichtBelegt = true;
				}
			}

		}
		
		if (eineNichtBelegt){
			return -1;
		}
		
		return 0;
	}
	
	public ArrayList<Variable> getVariablen(){
		/*
		 * Gibt die ArrayList<Variable> der Variablen zurück.
		 */
		return variablen;
	}
	

	public void belegeVariable(String name, int belegung){
		/*
		 * Belegt alle Variablen der Klausel mit Bezeichner "name" mit der Belegung "belegung".
		 */
		
		ListIterator<Variable> litr = variablen.listIterator();
		
		
		while (litr.hasNext()) {
			Variable element = litr.next();
			if (element.getBezeichner().equals(name)){
				element.setBelegung(belegung);
			}

		}
		
		
	}
	
	public void addVariable(Variable var){
		/*
		 * Fügt (deep copy der) Variable an letzter Stelle der Klausel hinzu.
		 */
		String bez = var.getBezeichner();
		int bel = var.getBelegung();
		boolean neg = var.getNegiert();
		Variable neu = new Variable(bez,neg);
		neu.setBelegung(bel);
		
		variablen.add(neu);
	}
	
	public HashSet<String> getVariablenOhneNegation(){
		/*
		 * Gibt alle einzigartigen Variabelnamen (ohne Negation) in der Klausel zurück, die noch nicht belegt sind.
		 */
		HashSet<String> res = new HashSet<String>();		
		
		ListIterator<Variable> litr = variablen.listIterator();
		
		while (litr.hasNext()) {
			Variable element = litr.next();
			if (element.getBelegung()== -1){
				String name = element.getBezeichner();
			
				res.add(name);
			}

		}
		
		return res;
		
	}
	
	public HashSet<String> getVariablenMitNegation(){
		/*
		 * Gibt alle einzigartigen Variabelnamen (einzigartig bis auf Negation, p und !p zählen als verschieden) in der Klausel zurück, die noch nicht belegt sind.
		 */
		HashSet<String> res = new HashSet<String>();		
		
		ListIterator<Variable> litr = variablen.listIterator();
		
		while (litr.hasNext()) {
			
			Variable element = litr.next();
//			System.out.println("Betrachte einzelne Variable : "+ element);
//			System.out.println("Betrachte einzelne Variable belegung: "+ element.getBelegung());
//			System.out.println("Betrachte einzelne Variable Name: "+ element.toString());
			if (element.getBelegung()== -1){
				String name = element.toString();
			
				res.add(name);
			}
		}
		
		return res;		
	}	
	
	public int getGroeße(){
		/*
		 * Gibt Anzahl der Variablen in der Klausel zurück.
		 */
		return variablen.size();
	}
	
	public String toString(){
		/*
		 * Gibt Klausel als String mit Klammern und Operatoren aus.
		 */
		String res = "(";		
		
		ListIterator<Variable> litr = variablen.listIterator();
		
		while (litr.hasNext()) {
			Variable element = litr.next();
			res += element.toString()+"\u2228";
		}
		
		
		return res.substring(0, res.length()-1)+")";	
	}

}
