import java.util.ArrayList;
import java.util.HashSet;
import java.util.ListIterator;


public class Klauselmenge{
	/*
	 * Klasse zur Verwaltung von mehreren KNF-Klauseln.
	 */
	private ArrayList<Klausel> klauseln  = new ArrayList<Klausel>();	
	
	public Klauselmenge(ArrayList<Klausel> kls){
		/*
		 * Konstruktor. Erstellt eine Klauselmenge mit einer deep copy der Klauseln.
		 */
		
		ListIterator<Klausel> litr = kls.listIterator();
		while (litr.hasNext()) {
			Klausel element = litr.next();
			Klausel neu = new Klausel(element.getVariablen());
			klauseln.add(neu);			
		}
		
		
	}
	
	public Klauselmenge(){}
	
	public boolean getIsTop(){
		/*
		 * Gibt zurück, ob Klauselmenge zu Top ausgewertet wird.
		 */
		return false;
	}
	
	public boolean getIsBottom(){
		/*
		 * Gibt zurück, ob Klauselmenge zu Bottom ausgewertet wird.
		 */
		return false;
	}
	
	public int eval(){
		/*
		 * Gibt Wahrheitswert 0, wenn eine Klausel die Klauselmenge nicht erfüllt (Konjunktion). 
		 * Falls mindestens eine Klausel zu -1 evaluiert und keine zu 0, wird -1 zurückgegeben.
		 * Sonst wird 1 zurückgegeben. 
		 */
		
		ListIterator<Klausel> litr = klauseln.listIterator();
		boolean eineNichtBelegt = false;
		
		while (litr.hasNext()) {
			Klausel element = litr.next();
			//System.out.println("Betrachte Variable "+element.toString()+"... Bewertung: "+element.eval());
			if (element.eval() == 0){
				
				return 0;
			} else{
				if (element.eval() == -1){
					eineNichtBelegt = true;
				}
			}

		}
		
		if (eineNichtBelegt){
			return -1;
		}
		
		return 1;
	}
	
	public ArrayList<Klausel> getKlauseln(){
		/*
		 * Gibt Klauseln der Klauselmenge als ArrayList<Klausel> zurück.
		 */
		return klauseln;
	}
	
	public void addKlausel(Klausel kl){
		/*
		 * Fügt eine deep copy der Klausel an letzter Stelle in die Klauselmenge ein.
		 */
		Klausel neu = new Klausel(kl.getVariablen());
		klauseln.add(neu);		
	}
	
	public HashSet<String> getVariablenOhneNegation(){
		/*
		 * Gibt alle einzigartigen Variabelnamen (ohne Negation) in der Klauselmenge zurück, die noch nicht belegt sind.
		 */
		HashSet<String> res = new HashSet<String>();		
		
		ListIterator<Klausel> litr = klauseln.listIterator();
		
		while (litr.hasNext()) {
			Klausel element = litr.next();
			HashSet<String> namen = element.getVariablenOhneNegation();			
			res.addAll(namen);
		}
		
		return res;
	}
	
	public HashSet<String> getVariablenMitNegation(){
		/*
		 * Gibt alle einzigartigen Variabelnamen (einzigartig bis auf Negation, p und !p zählen als verschieden) in der Klauselmenge zurück, die noch nicht belegt sind.
		 */
		HashSet<String> res = new HashSet<String>();		
		
		ListIterator<Klausel> litr = klauseln.listIterator();
		
		while (litr.hasNext()) {
			
			Klausel element = litr.next();
			HashSet<String> namen = element.getVariablenMitNegation();			
			res.addAll(namen);
		}
		
		return res;
	}
	
	public void belegeVariable(String name, int belegung){
		/*
		 * Belegt eine Variable in der Klauselmenge.
		 */
		
		System.out.println("Etwas will eine Variable belegen... : "+ name + ": " + belegung);
		
		ListIterator<Klausel> litr = klauseln.listIterator();
		
		while (litr.hasNext()) {
			Klausel element = litr.next();
			element.belegeVariable(name,belegung);
			
		}
	}
	
	public Klauselmenge copy(){
		/*
		 * Gibt eine Kopie der derzeitigen Klauselmenge zurück.
		 */
		Klauselmenge cop = new Klauselmenge(klauseln);
		return cop;
		
	}
	
	public Klauselmenge evalForm(){
		/*
		 * Falls Klauselmenge schon Top- oder Bottomklauselmenge ist, wird nichts gemacht.
		 * Gibt eine gekürtzte Form der Klauselmenge aus, in der zu 1 evaluierte Klauseln und zu 0 evaluierten Variablen in noch nicht erfüllten Klauseln gestirchen sind.
		 * Falls eine Klausel zu 0 evaluiert wurde, gibt evalForm() eine BOTTOM-Klauselmenge zurück, in der nur eine Klausel mit einer Variable BOTTOM ist, die immer zu 0 evaluiert wird. (d.h.
		 * die übergebene Klauselmenge ist mit der Belegung unerfüllbar)
		 * Falls am am Ende nach dem regelkonformen entfernen eine leere Klauselmenge ensteht, gib eine TOP-Klauselmenge zurück (Klauselmenge mit Belegung erfüllbar).
		 */
		
		if (this.getIsBottom()){
			return this;
		}
		
		if (this.getIsTop()){
			return this;
		}
		
		Klauselmenge res = new Klauselmenge(klauseln); //Kopie erstellen
		
		System.out.println("Klauselmenge, die gekürtzt werden soll: "+res.toString());
		ListIterator<Klausel> litr = res.getKlauseln().listIterator();
		
		while (litr.hasNext()) {
			Klausel element = litr.next();
			System.out.println("Betrachte element : "+element.toString());
			
			//Alle zu 1 evaluierten Klauseln entfernen
			if(element.eval()==1){
				litr.remove();				
			}			
			else{
				//Prüfe, ob eine Klausel zu 0 evaluiert wurde
				if(element.eval()==0){
					System.out.println("EVAL 0");
					return new Bottom();
				}
				
				
				ListIterator<Variable> litrb = element.getVariablen().listIterator();				
				while (litrb.hasNext()) {
					Variable var = litrb.next();					
					//Alle zu 0 evaluierten Variablen in der Klausel, die noch nicht zu 0 evaluiert wurde, entfernen
					
					if(var.eval()==0 ){
						litrb.remove();
					}else{
						
						
					}
				}
			}
		}
		
		
		//Falls nach entfernen eine leere Menge auftritt, gib Top- Klauselmenge zurück
		System.out.println("nach eval : "+res.getKlauseln().toString());
		if (res.getKlauseln().isEmpty()){	
			System.out.println("Leer : "+new Top().toString());
			return new Top();			
		}
		
		return res;
		
		
		
	}
	
	public Klauselmenge removeKlauselAtIndex(int index){
		/*
		 * Gibt neue Klauselmenge ohne die Klausel an Position index aus.
		 */
		Klauselmenge res = new Klauselmenge(klauseln); //Kopie erstellen		
		res.getKlauseln().remove(index);
		return res;
	}
	
	public String toString(){
		/*
		 * Gibt Klauselmenge als String mit Operatoren und Klammern aus.
		 */
		String res = "";		
		
		ListIterator<Klausel> litr = klauseln.listIterator();
		
		if (klauseln.isEmpty() ) {
			return "toString von einer nicht Bottom oder Top menge angewandt.";
		}
		
		while (litr.hasNext()) {
			Klausel element = litr.next();
			res += element.toString()+"\u2227";
		}
		
		return res.substring(0, res.length()-1);
	}
	
}

class Bottom extends Klauselmenge{
	ArrayList<Klausel> klauseln;
	
	public Bottom(){
		/*
		 * Eine vorgebaute Klauselmenge, bestehend aus nur einer Variable BOTTOM, die immer zu 0 evaluiert wird.
		 */
		Variable bot = new Variable("BOTTOM",false);
		bot.setBelegung(0);
		ArrayList<Variable> varList = new ArrayList<Variable>();
		varList.add(bot);
		Klausel botKlausel = new Klausel(varList);
		klauseln = new ArrayList<Klausel>();
		klauseln.add(botKlausel);
	}
	
	public String toString(){
		return "\u22A5";
	}
	
	public boolean getIsBottom(){
		return true;
	}
	
	
}

class Top extends Klauselmenge{
	ArrayList<Klausel> klauseln;
	public Top(){
		/*
		 * Eine vorgebaute Klauselmenge, bestehend aus nur einer Variable TOP, die immer zu 1 evaluiert wird.
		 */
		Variable bot = new Variable("Top",false);
		bot.setBelegung(1);
		ArrayList<Variable> varList = new ArrayList<Variable>();
		varList.add(bot);
		Klausel botKlausel = new Klausel(varList);
		klauseln = new ArrayList<Klausel>();
		klauseln.add(botKlausel);
	}
	
	public String toString(){
		return "\u22A4";
	}
	
	public boolean getIsTop(){
		return true;
	}
}
